// Copyright © 2018 TermySequence LLC
//
// SPDX-License-Identifier: GPL-2.0-only

#include "common.h"
#include "actions.h"
#include "customaction.h"
#include "base/manager.h"

static const QHash<QString,const char*> s_actionHash = {
    { A("AdjustTerminalColors"), ACT_ADJUST_TERMINAL_COLORS },
    { A("AdjustTerminalFont"), ACT_ADJUST_TERMINAL_FONT },
    { A("AdjustTerminalLayout"), ACT_ADJUST_TERMINAL_LAYOUT },
    { A("AdjustTerminalScrollback"), ACT_ADJUST_TERMINAL_SCROLLBACK },
    { A("AnnotateCommand"), ACT_ANNOTATE_COMMAND },
    { A("AnnotateLineContext"), ACT_ANNOTATE_LINE_CONTEXT },
    { A("AnnotateOutput"), ACT_ANNOTATE_OUTPUT },
    { A("AnnotateScreen"), ACT_ANNOTATE_SCREEN },
    { A("AnnotateScreen|0"), ACT_ANNOTATE_SCREEN_TOP },
    { A("AnnotateSelection"), ACT_ANNOTATE_SELECTION },
    { A("CancelTask"), ACT_CANCEL_TASK },
    { A("ClearAlert"), ACT_CLEAR_ALERT },
    { A("ClearTerminalScrollback"), ACT_CLEAR_TERMINAL_SCROLLBACK },
    { A("CloneTerminal"), ACT_CLONE_TERMINAL },
    { A("CloseTerminal"), ACT_CLOSE_TERMINAL },
    { A("CloseWindow"), ACT_CLOSE_WINDOW },
    { A("Copy"), ACT_COPY },
    { A("CopyCommand"), ACT_COPY_COMMAND },
    { A("CopyDirectoryPath"), ACT_COPY_DIRECTORY_PATH },
    { A("CopyFile"), ACT_COPY_FILE },
    { A("CopyFilePath"), ACT_COPY_FILE_PATH },
    { A("CopyJob"), ACT_COPY_JOB },
    { A("CopyOutput"), ACT_COPY_OUTPUT },
    { A("DecreaseFont"), ACT_DECREASE_FONT },
    { A("DeleteFile"), ACT_DELETE_FILE },
    { A("DisconnectServer"), ACT_DISCONNECT_SERVER },
    { A("DisconnectTerminal"), ACT_DISCONNECT_TERMINAL },
    { A("DownloadFile"), ACT_DOWNLOAD_FILE },
    { A("DownloadImage"), ACT_DOWNLOAD_IMAGE },
    { A("EditGlobalSettings"), ACT_EDIT_GLOBAL_SETTINGS },
    { A("EditIconRules"), ACT_EDIT_ICON_RULES },
    { A("EditKeymap"), ACT_EDIT_KEYMAP },
    { A("EditProfile"), ACT_EDIT_PROFILE },
    { A("EditServer"), ACT_EDIT_SERVER },
    { A("EditSwitchRules"), ACT_EDIT_SWITCH_RULES },
    { A("EventLog"), ACT_EVENT_LOG },
    { A("ExitFullScreen"), ACT_EXIT_FULL_SCREEN },
    { A("ExitPresentationMode"), ACT_EXIT_PRESENTATION_MODE },
    { A("ExtractProfile"), ACT_EXTRACT_PROFILE },
    { A("Find"), ACT_FIND },
    { A("HideServer"), ACT_HIDE_SERVER },
    { A("HideTerminal"), ACT_HIDE_TERMINAL },
    { A("HideTerminalEverywhere"), ACT_HIDE_TERMINAL_EVERYWHERE },
    { A("HighlightSemanticRegions"), ACT_HIGHLIGHT_SEMANTIC_REGIONS },
    { A("IncreaseFont"), ACT_INCREASE_FONT },
    { A("InputSetFollower"), ACT_INPUT_SET_FOLLOWER },
    { A("InputToggleFollower"), ACT_INPUT_TOGGLE_FOLLOWER },
    { A("InputUnsetFollower"), ACT_INPUT_UNSET_FOLLOWER },
    { A("InputSetLeader"), ACT_INPUT_SET_LEADER },
    { A("InputUnsetLeader"), ACT_INPUT_UNSET_LEADER },
    { A("InspectTask"), ACT_INSPECT_TASK },
    { A("ManageAlerts"), ACT_MANAGE_ALERTS },
    { A("ManageConnections"), ACT_MANAGE_CONNECTIONS },
    { A("ManageKeymaps"), ACT_MANAGE_KEYMAPS },
    { A("ManageLaunchers"), ACT_MANAGE_LAUNCHERS },
    { A("ManagePlugins"), ACT_MANAGE_PLUGINS },
    { A("ManagePortForwarding"), ACT_MANAGE_PORT_FORWARDING },
    { A("ManageProfiles"), ACT_MANAGE_PROFILES },
    { A("ManageServers"), ACT_MANAGE_SERVERS },
    { A("ManageTerminals"), ACT_MANAGE_TERMINALS },
    { A("NewLocalTerminal"), ACT_NEW_LOCAL_TERMINAL },
    { A("NewTerminal"), ACT_NEW_TERMINAL },
    { A("NewWindow"), ACT_NEW_WINDOW },
    { A("NextPane"), ACT_NEXT_PANE },
    { A("NextServer"), ACT_NEXT_SERVER },
    { A("NextTerminal"), ACT_NEXT_TERMINAL },
    { A("OpenFile"), ACT_OPEN_FILE },
    { A("OpenTaskDirectory"), ACT_OPEN_TASK_DIRECTORY },
    { A("OpenTaskFile"), ACT_OPEN_TASK_FILE },
    { A("OpenUrl"), ACT_OPEN_URL },
    { A("Paste"), ACT_PASTE },
    { A("PasteFile"), ACT_PASTE_FILE },
    { A("PasteSelectBuffer"), ACT_PASTE_SELECT_BUFFER },
    { A("PopProfile"), ACT_POP_PROFILE },
    { A("PreviousPane"), ACT_PREVIOUS_PANE },
    { A("PreviousServer"), ACT_PREVIOUS_SERVER },
    { A("PreviousTerminal"), ACT_PREVIOUS_TERMINAL },
    { A("Prompt"), ACT_PROMPT },
    { A("PushProfile"), ACT_PUSH_PROFILE },
    { A("QuitApplication"), ACT_QUIT_APPLICATION },
    { A("RaiseActiveTool"), ACT_RAISE_ACTIVE_TOOL },
    { A("RaiseAnnotationsTool"), ACT_RAISE_ANNOTATIONS_TOOL },
    { A("RaiseFilesTool"), ACT_RAISE_FILES_TOOL },
    { A("RaiseHistoryTool"), ACT_RAISE_HISTORY_TOOL },
    { A("RaiseKeymapTool"), ACT_RAISE_KEYMAP_TOOL },
    { A("RaiseSearchTool"), ACT_RAISE_SEARCH_TOOL },
    { A("RaiseSuggestionsTool"), ACT_RAISE_SUGGESTIONS_TOOL },
    { A("RaiseTasksTool"), ACT_RAISE_TASKS_TOOL },
    { A("RaiseTerminalsTool"), ACT_RAISE_TERMINALS_TOOL },
    { A("RandomTerminalTheme"), ACT_RANDOM_TERMINAL_THEME },
    { A("RemoveNote"), ACT_REMOVE_NOTE },
    { A("RemoveSuggestion"), ACT_REMOVE_COMMAND },
    { A("RemoveTasks"), ACT_REMOVE_TASKS },
    { A("RenameFile"), ACT_RENAME_FILE },
    { A("ReorderServerBackward"), ACT_REORDER_SERVER_BACKWARD },
    { A("ReorderServerFirst"), ACT_REORDER_SERVER_FIRST },
    { A("ReorderServerForward"), ACT_REORDER_SERVER_FORWARD },
    { A("ReorderServerLast"), ACT_REORDER_SERVER_LAST },
    { A("ReorderTerminalBackward"), ACT_REORDER_TERMINAL_BACKWARD },
    { A("ReorderTerminalFirst"), ACT_REORDER_TERMINAL_FIRST },
    { A("ReorderTerminalForward"), ACT_REORDER_TERMINAL_FORWARD },
    { A("ReorderTerminalLast"), ACT_REORDER_TERMINAL_LAST },
    { A("ResetAndClearTerminal"), ACT_RESET_AND_CLEAR_TERMINAL },
    { A("ResetTerminal"), ACT_RESET_TERMINAL },
    { A("RestartTask"), ACT_RESTART_TASK },
    { A("ScrollLineDown"), ACT_SCROLL_LINE_DOWN },
    { A("ScrollLineUp"), ACT_SCROLL_LINE_UP },
    { A("ScrollNoteDown"), ACT_SCROLL_NOTE_DOWN },
    { A("ScrollNoteUp"), ACT_SCROLL_NOTE_UP },
    { A("ScrollPageDown"), ACT_SCROLL_PAGE_DOWN },
    { A("ScrollPageUp"), ACT_SCROLL_PAGE_UP },
    { A("ScrollPromptDown"), ACT_SCROLL_PROMPT_DOWN },
    { A("ScrollPromptFirst"), ACT_SCROLL_PROMPT_FIRST },
    { A("ScrollPromptLast"), ACT_SCROLL_PROMPT_LAST },
    { A("ScrollPromptUp"), ACT_SCROLL_PROMPT_UP },
    { A("ScrollToBottom"), ACT_SCROLL_TO_BOTTOM },
    { A("ScrollToTop"), ACT_SCROLL_TO_TOP },
    { A("SearchDown"), ACT_SEARCH_DOWN },
    { A("SearchReset"), ACT_SEARCH_RESET },
    { A("SearchUp"), ACT_SEARCH_UP },
    { A("SelectAll"), ACT_SELECT_ALL },
    { A("SelectCommand"), ACT_SELECT_COMMAND },
    { A("SelectHandle"), ACT_SELECT_HANDLE },
    { A("SelectHandleBackChar"), ACT_SELECT_HANDLE_BACK_CHAR },
    { A("SelectHandleBackWord"), ACT_SELECT_HANDLE_BACK_WORD },
    { A("SelectHandleDownLine"), ACT_SELECT_HANDLE_DOWN_LINE },
    { A("SelectHandleForwardChar"), ACT_SELECT_HANDLE_FORWARD_CHAR },
    { A("SelectHandleForwardWord"), ACT_SELECT_HANDLE_FORWARD_WORD },
    { A("SelectHandleUpLine"), ACT_SELECT_HANDLE_UP_LINE },
    { A("SelectJob"), ACT_SELECT_JOB },
    { A("SelectLine"), ACT_SELECT_LINE },
    { A("SelectLine|1"), ACT_SELECT_LINE_START },
    { A("SelectLine|2"), ACT_SELECT_LINE_END },
    { A("SelectMoveBackWord"), ACT_SELECT_MOVE_BACK_WORD },
    { A("SelectMoveDownLine"), ACT_SELECT_MOVE_DOWN_LINE },
    { A("SelectMoveForwardWord"), ACT_SELECT_MOVE_FORWARD_WORD },
    { A("SelectMoveUpLine"), ACT_SELECT_MOVE_UP_LINE },
    { A("SelectOutput"), ACT_SELECT_OUTPUT },
    { A("SelectScreen"), ACT_SELECT_SCREEN },
    { A("SelectWord"), ACT_SELECT_WORD },
    { A("SetFileListingFormat"), ACT_SET_FILE_LISTING_FORMAT },
    { A("SetServerIcon"), ACT_SET_SERVER_ICON },
    { A("SetServerIcon|"), ACT_SET_SERVER_ICON_DEFAULT },
    { A("SetTerminalIcon"), ACT_SET_TERMINAL_ICON },
    { A("SetTerminalIcon|"), ACT_SET_TERMINAL_ICON_DEFAULT },
    { A("SetTerminalIcon|none"), ACT_SET_TERMINAL_ICON_EMPTY },
    { A("SetupTasks"), ACT_SETUP_TASKS },
    { A("ShowMenuBar"), ACT_SHOW_MENU_BAR },
    { A("ShowServer"), ACT_SHOW_SERVER },
    { A("SplitViewClose"), ACT_SPLIT_VIEW_CLOSE },
    { A("SplitViewCloseOthers"), ACT_SPLIT_VIEW_CLOSE_OTHERS },
    { A("SplitViewEqualize"), ACT_SPLIT_VIEW_EQUALIZE },
    { A("SplitViewEqualizeAll"), ACT_SPLIT_VIEW_EQUALIZE_ALL },
    { A("SplitViewExpand"), ACT_SPLIT_VIEW_EXPAND },
    { A("SplitViewHorizontal"), ACT_SPLIT_VIEW_HORIZONTAL },
    { A("SplitViewHorizontalFixed"), ACT_SPLIT_VIEW_HORIZONTAL_FIXED },
    { A("SplitViewQuadFixed"), ACT_SPLIT_VIEW_QUAD_FIXED },
    { A("SplitViewShrink"), ACT_SPLIT_VIEW_SHRINK },
    { A("SplitViewVertical"), ACT_SPLIT_VIEW_VERTICAL },
    { A("SplitViewVerticalFixed"), ACT_SPLIT_VIEW_VERTICAL_FIXED },
    { A("SuggestNext"), ACT_SUGGEST_NEXT },
    { A("SuggestPrevious"), ACT_SUGGEST_PREVIOUS },
    { A("SwitchPane"), ACT_SWITCH_PANE },
    { A("TakeTerminalOwnership"), ACT_TAKE_TERMINAL_OWNERSHIP },
    { A("TerminalContextMenu"), ACT_TERMINAL_CONTEXT_MENU },
    { A("TimingFloatOrigin"), ACT_TIMING_FLOAT_ORIGIN },
    { A("TimingSetOriginContext"), ACT_TIMING_SET_ORIGIN_CONTEXT },
    { A("ToggleAnnotationsTool"), ACT_TOGGLE_ANNOTATIONS_TOOL },
    { A("ToggleCommandMode"), ACT_TOGGLE_COMMAND_MODE },
    { A("ToggleFilesTool"), ACT_TOGGLE_FILES_TOOL },
    { A("ToggleFullScreen"), ACT_TOGGLE_FULL_SCREEN },
    { A("ToggleHistoryTool"), ACT_TOGGLE_HISTORY_TOOL },
    { A("ToggleKeymapTool"), ACT_TOGGLE_KEYMAP_TOOL },
    { A("ToggleMenuBar"), ACT_TOGGLE_MENU_BAR },
    { A("TogglePresentationMode"), ACT_TOGGLE_PRESENTATION_MODE },
    { A("ToggleSearchTool"), ACT_TOGGLE_SEARCH_TOOL },
    { A("ToggleSelectionMode"), ACT_TOGGLE_SELECTION_MODE },
    { A("ToggleSoftScrollLock"), ACT_TOGGLE_SOFT_SCROLL_LOCK },
    { A("ToggleStatusBar"), ACT_TOGGLE_STATUS_BAR },
    { A("ToggleSuggestionsTool"), ACT_TOGGLE_SUGGESTIONS_TOOL },
    { A("ToggleTasksTool"), ACT_TOGGLE_TASKS_TOOL },
    { A("ToggleTerminalFollowing"), ACT_TOGGLE_TERMINAL_FOLLOWING },
    { A("ToggleTerminalLayout"), ACT_TOGGLE_TERMINAL_LAYOUT },
    { A("ToggleTerminalRemoteInput"), ACT_TOGGLE_TERMINAL_REMOTE_INPUT },
    { A("ToggleTerminalsTool"), ACT_TOGGLE_TERMINALS_TOOL },
    { A("ToggleToolSearchBar"), ACT_TOGGLE_TOOL_SEARCH_BAR },
    { A("ToggleToolTableHeader"), ACT_TOGGLE_TOOL_TABLE_HEADER },
    { A("ToolContextMenu"), ACT_TOOL_CONTEXT_MENU },
    { A("ToolAction|0"), ACT_TOOL_ACTION0 },
    { A("ToolAction|1"), ACT_TOOL_ACTION1 },
    { A("ToolAction|2"), ACT_TOOL_ACTION2 },
    { A("ToolAction|3"), ACT_TOOL_ACTION3 },
    { A("ToolFilterAddServer"), ACT_TOOL_FILTER_ADD_SERVER },
    { A("ToolFilterAddTerminal"), ACT_TOOL_FILTER_ADD_TERMINAL },
    { A("ToolFilterExcludeServer"), ACT_TOOL_FILTER_EXCLUDE_SERVER },
    { A("ToolFilterExcludeTerminal"), ACT_TOOL_FILTER_EXCLUDE_TERMINAL },
    { A("ToolFilterIncludeNothing"), ACT_TOOL_FILTER_INCLUDE_NOTHING },
    { A("ToolFilterRemoveClosed"), ACT_TOOL_FILTER_REMOVE_CLOSED },
    { A("ToolFilterReset"), ACT_TOOL_FILTER_RESET },
    { A("ToolFilterSetServer"), ACT_TOOL_FILTER_SET_SERVER },
    { A("ToolFilterSetTerminal"), ACT_TOOL_FILTER_SET_TERMINAL },
    { A("ToolFirst"), ACT_TOOL_FIRST },
    { A("ToolLast"), ACT_TOOL_LAST },
    { A("ToolNext"), ACT_TOOL_NEXT },
    { A("ToolPrevious"), ACT_TOOL_PREVIOUS },
    { A("ToolSearch"), ACT_TOOL_SEARCH },
    { A("ToolSearchReset"), ACT_TOOL_SEARCH_RESET },
    { A("UndoAllAdjustments"), ACT_UNDO_ALL_ADJUSTMENTS },
    { A("UploadFile"), ACT_UPLOAD_FILE },
    { A("UploadToDirectory"), ACT_UPLOAD_DIR },
    { A("ViewServerInfo"), ACT_VIEW_SERVER_INFO },
    { A("ViewTerminalContent"), ACT_VIEW_TERMINAL_CONTENT },
    { A("ViewTerminalInfo"), ACT_VIEW_TERMINAL_INFO },
    { A("WriteCommand"), ACT_WRITE_COMMAND },
    { A("WriteCommandNewline"), ACT_WRITE_COMMAND_NEWLINE },
    { A("WriteDirectoryPath"), ACT_WRITE_DIRECTORY_PATH },
    { A("WriteFilePath"), ACT_WRITE_FILE_PATH },
    { A("WriteSelection"), ACT_WRITE_SELECTION },
    { A("WriteSuggestion"), ACT_WRITE_COMMAND },
    { A("WriteSuggestion|0"), ACT_WRITE_COMMAND_N },
    { A("WriteSuggestion|1"), ACT_WRITE_COMMAND_N },
    { A("WriteSuggestion|2"), ACT_WRITE_COMMAND_N },
    { A("WriteSuggestion|3"), ACT_WRITE_COMMAND_N },
};

QString
actionString(QString slot)
{
    const char *result = ACT_NONE;

    auto i = s_actionHash.constFind(slot);
    if (i != s_actionHash.cend()) {
        result = *i;
        goto out;
    }

    slot = slot.left(slot.indexOf('|'));
    i = s_actionHash.constFind(slot);
    if (i != s_actionHash.cend()) {
        result = *i;
        goto out;
    }

    if (slot.startsWith(CUSTOM_PREFIX)) {
        slot.remove(0, CUSTOM_PREFIX_LEN);
        slot = ActionFeature::getDescription(slot);
        if (!slot.isEmpty())
            return slot;
    }

out:
    return QCoreApplication::translate("action", result);
}
