// Copyright © 2018 TermySequence LLC
//
// SPDX-License-Identifier: GPL-2.0-only

#include "common.h"
#include "attrbase.h"
#include "defmap.h"

using namespace Tsqt;
using namespace Tsq;

// NOTE: digraphs are not supported in this keymap

const KeymapRule g_defaultKeymap[] = {
    { SelectMode, SelectMode, ' ', 0, 0, 1, A("Space"), B("SelectHandle") },
    { Control, Control, '+', 0, 0, 1, A("Plus"), B("IncreaseFont") },
    { Control, Control, '-', 0, 0, 1, A("Minus"), B("DecreaseFont") },
    { Alt, Alt, '1', 0, 0, 1, C('1'), B("SwitchPane|0") },
    { Alt, Alt, '2', 0, 0, 1, C('2'), B("SwitchPane|1") },
    { Alt, Alt, '3', 0, 0, 1, C('3'), B("SwitchPane|2") },
    { Alt, Alt, '4', 0, 0, 1, C('4'), B("SwitchPane|3") },
    { Control, Control, '1', 0, 0, 1, C('1'), B("WriteSuggestion|0") },
    { Control, Control, '2', 0, 0, 1, C('2'), B("WriteSuggestion|1") },
    { Control, Control, '3', 0, 0, 1, C('3'), B("WriteSuggestion|2") },
    { Control, Control, '4', 0, 0, 1, C('4'), B("WriteSuggestion|3") },
    { SelectMode, SelectMode, '1', 0, 0, 1, C('1'), B("SelectWord|0") },
    { SelectMode, SelectMode, '2', 0, 0, 1, C('2'), B("SelectWord|1") },
    { SelectMode, SelectMode, '3', 0, 0, 1, C('3'), B("SelectWord|2") },
    { SelectMode, SelectMode, '4', 0, 0, 1, C('4'), B("SelectWord|3") },
    { Control, Control, ';', 0, 0, 1, A("Semicolon"), B("SelectOutput") },
    { Shift|Control, Shift|Control, 'A', 0, 0, 1, C('A'), B("SelectAll") },
    { Control, Control, 'A', 0, 0, 0, C('A'), B("\x01") },
    { SelectMode, SelectMode, 'A', 0, 0, 1, C('A'), B("SelectHandleBackChar") },
    { Control, Control, 'B', 0, 0, 0, C('B'), B("\x02") },
    { Shift|Control, Shift|Control, 'C', 0, 0, 1, C('C'), B("Copy") },
    { Control, Control, 'C', 0, 0, 0, C('C'), B("\x03") },
    { Shift|Control, Shift|Control, 'D', 0, 0, 1, C('D'), B("SearchDown") },
    { Control, Control, 'D', 0, 0, 0, C('D'), B("\x04") },
    { SelectMode, SelectMode, 'D', 0, 0, 1, C('D'), B("SelectHandleForwardChar") },
    { Control, Control, 'E', 0, 0, 0, C('E'), B("\x05") },
    { SelectMode, SelectMode, 'E', 0, 0, 1, C('E'), B("SelectHandleForwardWord") },
    { Shift|Control, Shift|Control, 'F', 0, 0, 1, C('F'), B("Find") },
    { Control, Control, 'F', 0, 0, 0, C('F'), B("\x06") },
    { Control, Control, 'G', 0, 0, 0, C('G'), B("\x07") },
    { Control, Control, 'H', 0, 0, 0, C('H'), B("\x08") },
    { Control, Control, 'I', 0, 0, 0, C('I'), B("\x09") },
    { Control, Control, 'J', 0, 0, 0, C('J'), B("\x0a") },
    { Control, Control, 'K', 0, 0, 0, C('K'), B("\x0b") },
    { Shift|Control, Shift|Control, 'L', 0, 0, 1, C('L'), B("NewLocalTerminal") },
    { Control, Control, 'L', 0, 0, 0, C('L'), B("\x0c") },
    { Control, Control, 'M', 0, 0, 0, C('M'), B("\x0d") },
    { Shift|Control, Shift|Control, 'N', 0, 0, 1, C('N'), B("NewWindow") },
    { Control, Control, 'N', 0, 0, 0, C('N'), B("\x0e") },
    { Control, Control, 'O', 0, 0, 0, C('O'), B("\x0f") },
    { Control, Control, 'P', 0, 0, 0, C('P'), B("\x10") },
    { Shift|Control, Shift|Control, 'Q', 0, 0, 1, C('Q'), B("CloseWindow") },
    { Control, Control, 'Q', 0, 0, 0, C('Q'), B("\x11") },
    { SelectMode, SelectMode, 'Q', 0, 0, 1, C('Q'), B("SelectHandleBackWord") },
    { Shift|Control, Shift|Control, 'R', 0, 0, 1, C('R'), B("SearchReset") },
    { Control, Control, 'R', 0, 0, 0, C('R'), B("\x12") },
    { Shift|Control, Shift|Control, 'S', 0, 0, 1, C('S'), B("ToolSearch") },
    { Control, Control, 'S', 0, 0, 0, C('S'), B("\x13") },
    { SelectMode, SelectMode, 'S', 0, 0, 1, C('S'), B("SelectHandleDownLine") },
    { Shift|Control, Shift|Control, 'T', 0, 0, 1, C('T'), B("NewTerminal") },
    { Control, Control, 'T', 0, 0, 0, C('T'), B("\x14") },
    { Shift|Control, Shift|Control, 'U', 0, 0, 1, C('U'), B("SearchUp") },
    { Control, Control, 'U', 0, 0, 0, C('U'), B("\x15") },
    { Shift|Control, Shift|Control, 'V', 0, 0, 1, C('V'), B("Paste") },
    { Control, Control, 'V', 0, 0, 0, C('V'), B("\x16") },
    { Shift|Control, Shift|Control, 'W', 0, 0, 1, C('W'), B("CloseTerminal") },
    { Control, Control, 'W', 0, 0, 0, C('W'), B("\x17") },
    { SelectMode, SelectMode, 'W', 0, 0, 1, C('W'), B("SelectHandleUpLine") },
    { Control, Control, 'X', 0, 0, 0, C('X'), B("\x18") },
    { Control, Control, 'Y', 0, 0, 0, C('Y'), B("\x19") },
    { Control, Control, 'Z', 0, 0, 0, C('Z'), B("\x1a") },
    { SelectMode, SelectMode, 0x01000000, 0, 0, 1, A("Escape"), B("ToggleSelectionMode") },
    { CommandMode, CommandMode, 0x01000000, 0, 0, 1, A("Escape"), B("ToggleCommandMode") },
    { None, None, 0x01000000, 0, 0, 0, A("Escape"), B("\x1b") },
    { Control, Control, 0x01000001, 0, 0, 1, A("Tab"), B("SuggestPrevious") },
    { None, None, 0x01000001, 0, 0, 0, A("Tab"), B("\t") },
    { None, None, 0x01000002, 0, 0, 1, A("Backtab"), B("SuggestNext") },
    { None, None, 0x01000003, 0, 0, 0, A("Backspace"), B("\x7f") },
    { SelectMode|Shift, SelectMode|Shift, 0x01000004, 0, 0, 1, A("Return"), B("WriteSelection") },
    { Shift, Shift, 0x01000004, 0, 0, 1, A("Return"), B("WriteSuggestion") },
    { Control, Control, 0x01000004, 0, 0, 1, A("Return"), B("ToolAction|0") },
    { Alt, Alt, 0x01000004, 0, 0, 1, A("Return"), B("ToolAction|1") },
    { NewLine, NewLine, 0x01000004, 0, 0, 0, A("Return"), B("\r\n") },
    { None, NewLine, 0x01000004, 0, 0, 0, A("Return"), B("\r") },
    { NewLine, NewLine, 0x01000005, 0, 0, 0, A("Enter"), B("\r\n") },
    { None, NewLine, 0x01000005, 0, 0, 0, A("Enter"), B("\r") },
    { Shift|Control, Shift|Control, 0x01000006, 0, 0, 1, A("Insert"), B("PasteSelectBuffer") },
    { Shift, Shift, 0x01000006, 0, 0, 1, A("Insert"), B("Paste") },
    { SelectMode, SelectMode, 0x01000006, 0, 0, 1, A("Insert"), B("SelectLine") },
    { AnyMod, AnyMod, 0x01000006, 0, 0, 0, A("Insert"), B("\x1b[2;*~") },
    { None, None, 0x01000006, 0, 0, 0, A("Insert"), B("\x1b[2~") },
    { AnyMod, AnyMod, 0x01000007, 0, 0, 0, A("Delete"), B("\x1b[3;*~") },
    { None, None, 0x01000007, 0, 0, 0, A("Delete"), B("\x1b[3~") },
    { Alt, Alt, 0x01000010, 0, 0, 1, A("Home"), B("ToolFirst") },
    { Shift, AppScreen|Shift, 0x01000010, 0, 0, 1, A("Home"), B("ScrollToTop") },
    { SelectMode, SelectMode, 0x01000010, 0, 0, 1, A("Home"), B("SelectLine|1") },
    { AppCuKeys, AppCuKeys, 0x01000010, 0, 0, 0, A("Home"), B("\x1bOH") },
    { AnyMod, AnyMod, 0x01000010, 0, 0, 0, A("Home"), B("\x1b[1;*H") },
    { None, None, 0x01000010, 0, 0, 0, A("Home"), B("\x1b[H") },
    { Alt, Alt, 0x01000011, 0, 0, 1, A("End"), B("ToolLast") },
    { Shift, AppScreen|Shift, 0x01000011, 0, 0, 1, A("End"), B("ScrollToBottom") },
    { SelectMode, SelectMode, 0x01000011, 0, 0, 1, A("End"), B("SelectLine|2") },
    { AppCuKeys, AppCuKeys, 0x01000011, 0, 0, 0, A("End"), B("\x1bOF") },
    { AnyMod, AnyMod, 0x01000011, 0, 0, 0, A("End"), B("\x1b[1;*F") },
    { None, None, 0x01000011, 0, 0, 0, A("End"), B("\x1b[F") },
    { Shift|Control, Shift|Control, 0x01000012, 0, 0, 1, A("Left"), B("ReorderTerminalForward") },
    { Control, Control, 0x01000012, 0, 0, 1, A("Left"), B("PreviousPane") },
    { Shift, Shift, 0x01000012, 0, 0, 1, A("Left"), B("PreviousTerminal") },
    { SelectMode, SelectMode, 0x01000012, 0, 0, 1, A("Left"), B("SelectMoveBackWord") },
    { AppCuKeys, AppCuKeys, 0x01000012, 0, 0, 0, A("Left"), B("\x1bOD") },
    { AnyMod, AnyMod, 0x01000012, 0, 0, 0, A("Left"), B("\x1b[1;*D") },
    { None, None, 0x01000012, 0, 0, 0, A("Left"), B("\x1b[D") },
    { Alt, Alt, 0x01000013, 0, 0, 1, A("Up"), B("ToolPrevious") },
    { Shift|Control, Shift|Control, 0x01000013, 0, 0, 1, A("Up"), B("ScrollPromptUp") },
    { Shift, AppScreen|Shift, 0x01000013, 0, 0, 1, A("Up"), B("ScrollLineUp") },
    { SelectMode, SelectMode, 0x01000013, 0, 0, 1, A("Up"), B("SelectMoveUpLine") },
    { AppCuKeys, AppCuKeys, 0x01000013, 0, 0, 0, A("Up"), B("\x1bOA") },
    { AnyMod, AnyMod, 0x01000013, 0, 0, 0, A("Up"), B("\x1b[1;*A") },
    { None, None, 0x01000013, 0, 0, 0, A("Up"), B("\x1b[A") },
    { Shift|Control, Shift|Control, 0x01000014, 0, 0, 1, A("Right"), B("ReorderTerminalBackward") },
    { Control, Control, 0x01000014, 0, 0, 1, A("Right"), B("NextPane") },
    { Shift, Shift, 0x01000014, 0, 0, 1, A("Right"), B("NextTerminal") },
    { SelectMode, SelectMode, 0x01000014, 0, 0, 1, A("Right"), B("SelectMoveForwardWord") },
    { AppCuKeys, AppCuKeys, 0x01000014, 0, 0, 0, A("Right"), B("\x1bOC") },
    { AnyMod, AnyMod, 0x01000014, 0, 0, 0, A("Right"), B("\x1b[1;*C") },
    { None, None, 0x01000014, 0, 0, 0, A("Right"), B("\x1b[C") },
    { Alt, Alt, 0x01000015, 0, 0, 1, A("Down"), B("ToolNext") },
    { Shift|Control, Shift|Control, 0x01000015, 0, 0, 1, A("Down"), B("ScrollPromptDown") },
    { Shift, AppScreen|Shift, 0x01000015, 0, 0, 1, A("Down"), B("ScrollLineDown") },
    { SelectMode, SelectMode, 0x01000015, 0, 0, 1, A("Down"), B("SelectMoveDownLine") },
    { AppCuKeys, AppCuKeys, 0x01000015, 0, 0, 0, A("Down"), B("\x1bOB") },
    { AnyMod, AnyMod, 0x01000015, 0, 0, 0, A("Down"), B("\x1b[1;*B") },
    { None, None, 0x01000015, 0, 0, 0, A("Down"), B("\x1b[B") },
    { Shift, AppScreen|Shift, 0x01000016, 0, 0, 1, A("PgUp"), B("ScrollPageUp") },
    { AnyMod, AnyMod, 0x01000016, 0, 0, 0, A("PgUp"), B("\x1b[5;*~") },
    { None, None, 0x01000016, 0, 0, 0, A("PgUp"), B("\x1b[5~") },
    { Shift, AppScreen|Shift, 0x01000017, 0, 0, 1, A("PgDown"), B("ScrollPageDown") },
    { AnyMod, AnyMod, 0x01000017, 0, 0, 0, A("PgDown"), B("\x1b[6;*~") },
    { None, None, 0x01000017, 0, 0, 0, A("PgDown"), B("\x1b[6~") },
    { None, None, 0x01000026, 0, 0, 1, A("ScrollLock"), B("ToggleSoftScrollLock") },
    { Shift, Shift, 0x01000030, 0, 0, 1, A("F1"), B("RaiseKeymapTool") },
    { Shift, Shift, 0x01000031, 0, 0, 1, A("F2"), B("RaiseTerminalsTool") },
    { Shift, Shift, 0x01000032, 0, 0, 1, A("F3"), B("RaiseSuggestionsTool") },
    { Shift, Shift, 0x01000033, 0, 0, 1, A("F4"), B("RaiseSearchTool") },
    { Shift, Shift, 0x01000034, 0, 0, 1, A("F5"), B("RaiseFilesTool") },
    { Shift, Shift, 0x01000035, 0, 0, 1, A("F6"), B("RaiseHistoryTool") },
    { Shift, Shift, 0x01000036, 0, 0, 1, A("F7"), B("RaiseAnnotationsTool") },
    { Shift, Shift, 0x01000037, 0, 0, 1, A("F8"), B("RaiseTasksTool") },
    { AnyMod, AnyMod, 0x01000030, 0, 0, 0, A("F1"), B("\x1b[1;*P") },
    { AnyMod, AnyMod, 0x01000031, 0, 0, 0, A("F2"), B("\x1b[1;*Q") },
    { AnyMod, AnyMod, 0x01000032, 0, 0, 0, A("F3"), B("\x1b[1;*R") },
    { AnyMod, AnyMod, 0x01000033, 0, 0, 0, A("F4"), B("\x1b[1;*S") },
    { AnyMod, AnyMod, 0x01000034, 0, 0, 0, A("F5"), B("\x1b[15;*~") },
    { AnyMod, AnyMod, 0x01000035, 0, 0, 0, A("F6"), B("\x1b[17;*~") },
    { AnyMod, AnyMod, 0x01000036, 0, 0, 0, A("F7"), B("\x1b[18;*~") },
    { AnyMod, AnyMod, 0x01000037, 0, 0, 0, A("F8"), B("\x1b[19;*~") },
    { AnyMod, AnyMod, 0x01000038, 0, 0, 0, A("F9"), B("\x1b[20;*~") },
    { AnyMod, AnyMod, 0x01000039, 0, 0, 0, A("F10"), B("\x1b[21;*~") },
    { AnyMod, AnyMod, 0x0100003a, 0, 0, 0, A("F11"), B("\x1b[23;*~") },
    { AnyMod, AnyMod, 0x0100003b, 0, 0, 0, A("F12"), B("\x1b[24;*~") },
    { None, None, 0x01000030, 0, 0, 0, A("F1"), B("\x1bOP") },
    { None, None, 0x01000031, 0, 0, 0, A("F2"), B("\x1bOQ") },
    { None, None, 0x01000032, 0, 0, 0, A("F3"), B("\x1bOR") },
    { None, None, 0x01000033, 0, 0, 0, A("F4"), B("\x1bOS") },
    { None, None, 0x01000034, 0, 0, 0, A("F5"), B("\x1b[15~") },
    { None, None, 0x01000035, 0, 0, 0, A("F6"), B("\x1b[17~") },
    { None, None, 0x01000036, 0, 0, 0, A("F7"), B("\x1b[18~") },
    { None, None, 0x01000037, 0, 0, 0, A("F8"), B("\x1b[19~") },
    { None, None, 0x01000038, 0, 0, 0, A("F9"), B("\x1b[20~") },
    { None, None, 0x01000039, 0, 0, 0, A("F10"), B("\x1b[21~") },
    // { None, None, 0x0100003a, 0, 0, 0, A("F11"), B("\x1b[23~") },
    // { None, None, 0x0100003b, 0, 0, 0, A("F12"), B("\x1b[24~") },
    { None, None, 0x0100003a, 0, 0, 1, A("F11"), B("ToggleFullScreen") },
    { None, None, 0x0100003b, 0, 0, 1, A("F12"), B("TogglePresentationMode") },
    { Shift, Shift, 0x01000055, 0, 0, 1, A("Menu"), B("ToolContextMenu") },
    { None, None, 0x01000055, 0, 0, 1, A("Menu"), B("TerminalContextMenu") },
    { None, None, 0x01000061, 0, 0, 1, A("Back"), B("PreviousTerminal") },
    { None, None, 0x01000062, 0, 0, 1, A("Forward"), B("NextTerminal") },
    { None, None }
};
