// Copyright © 2018 TermySequence LLC
//
// SPDX-License-Identifier: GPL-2.0-only

#include "common.h"
#include "attrbase.h"
#include "keys.h"

const QHash<QString,int> g_keyNames = {
    // Keys
    { A("Escape"), 0x01000000 },
    { A("Tab"), 0x01000001 },
    { A("Backtab"), 0x01000002 },
    { A("Backspace"), 0x01000003 },
    { A("Return"), 0x01000004 },
    { A("Enter"), 0x01000005 },
    { A("Insert"), 0x01000006 },
    { A("Delete"), 0x01000007 },
    { A("Pause"), 0x01000008 },
    { A("Print"), 0x01000009 },
    { A("SysReq"), 0x0100000a },
    { A("Clear"), 0x0100000b },
    { A("Home"), 0x01000010 },
    { A("End"), 0x01000011 },
    { A("Left"), 0x01000012 },
    { A("Up"), 0x01000013 },
    { A("Right"), 0x01000014 },
    { A("Down"), 0x01000015 },
    { A("PgUp"), 0x01000016 },
    { A("PgDown"), 0x01000017 },
//    { A("Shift"), 0x01000020 },
//    { A("Control"), 0x01000021 },
//    { A("Meta"), 0x01000022 },
//    { A("Alt"), 0x01000023 },
    { A("CapsLock"), 0x01000024 },
    { A("NumLock"), 0x01000025 },
    { A("ScrollLock"), 0x01000026 },
    { A("F1"), 0x01000030 },
    { A("F2"), 0x01000031 },
    { A("F3"), 0x01000032 },
    { A("F4"), 0x01000033 },
    { A("F5"), 0x01000034 },
    { A("F6"), 0x01000035 },
    { A("F7"), 0x01000036 },
    { A("F8"), 0x01000037 },
    { A("F9"), 0x01000038 },
    { A("F10"), 0x01000039 },
    { A("F11"), 0x0100003a },
    { A("F12"), 0x0100003b },
    { A("F13"), 0x0100003c },
    { A("F14"), 0x0100003d },
    { A("F15"), 0x0100003e },
    { A("F16"), 0x0100003f },
    { A("F17"), 0x01000040 },
    { A("F18"), 0x01000041 },
    { A("F19"), 0x01000042 },
    { A("F20"), 0x01000043 },
    { A("F21"), 0x01000044 },
    { A("F22"), 0x01000045 },
    { A("F23"), 0x01000046 },
    { A("F24"), 0x01000047 },
    { A("F25"), 0x01000048 },
    { A("F26"), 0x01000049 },
    { A("F27"), 0x0100004a },
    { A("F28"), 0x0100004b },
    { A("F29"), 0x0100004c },
    { A("F30"), 0x0100004d },
    { A("F31"), 0x0100004e },
    { A("F32"), 0x0100004f },
    { A("F33"), 0x01000050 },
    { A("F34"), 0x01000051 },
    { A("F35"), 0x01000052 },
    { A("Super_L"), 0x01000053 },
    { A("Super_R"), 0x01000054 },
    { A("Menu"), 0x01000055 },
    { A("Hyper_L"), 0x01000056 },
    { A("Hyper_R"), 0x01000057 },
    { A("Help"), 0x01000058 },
    { A("Direction_L"), 0x01000059 },
    { A("Direction_R"), 0x01000060 },
    { A("Space"), 0x20 },
    { A("Exclam"), 0x21 },
    { A("QuoteDbl"), 0x22 },
    { A("NumberSign"), 0x23 },
    { A("Dollar"), 0x24 },
    { A("Percent"), 0x25 },
    { A("Ampersand"), 0x26 },
    { A("Apostrophe"), 0x27 },
    { A("ParenLeft"), 0x28 },
    { A("ParenRight"), 0x29 },
    { A("Asterisk"), 0x2a },
    { A("Plus"), 0x2b },
    { A("Comma"), 0x2c },
    { A("Minus"), 0x2d },
    { A("Period"), 0x2e },
    { A("Slash"), 0x2f },
    { A("0"), 0x30 },
    { A("1"), 0x31 },
    { A("2"), 0x32 },
    { A("3"), 0x33 },
    { A("4"), 0x34 },
    { A("5"), 0x35 },
    { A("6"), 0x36 },
    { A("7"), 0x37 },
    { A("8"), 0x38 },
    { A("9"), 0x39 },
    { A("Colon"), 0x3a },
    { A("Semicolon"), 0x3b },
    { A("Less"), 0x3c },
    { A("Equal"), 0x3d },
    { A("Greater"), 0x3e },
    { A("Question"), 0x3f },
    { A("At"), 0x40 },
    { A("A"), 0x41 },
    { A("B"), 0x42 },
    { A("C"), 0x43 },
    { A("D"), 0x44 },
    { A("E"), 0x45 },
    { A("F"), 0x46 },
    { A("G"), 0x47 },
    { A("H"), 0x48 },
    { A("I"), 0x49 },
    { A("J"), 0x4a },
    { A("K"), 0x4b },
    { A("L"), 0x4c },
    { A("M"), 0x4d },
    { A("N"), 0x4e },
    { A("O"), 0x4f },
    { A("P"), 0x50 },
    { A("Q"), 0x51 },
    { A("R"), 0x52 },
    { A("S"), 0x53 },
    { A("T"), 0x54 },
    { A("U"), 0x55 },
    { A("V"), 0x56 },
    { A("W"), 0x57 },
    { A("X"), 0x58 },
    { A("Y"), 0x59 },
    { A("Z"), 0x5a },
    { A("BracketLeft"), 0x5b },
    { A("Backslash"), 0x5c },
    { A("BracketRight"), 0x5d },
    { A("AsciiCircum"), 0x5e },
    { A("Underscore"), 0x5f },
    { A("QuoteLeft"), 0x60 },
    { A("BraceLeft"), 0x7b },
    { A("Bar"), 0x7c },
    { A("BraceRight"), 0x7d },
    { A("AsciiTilde"), 0x7e },
    { A("NoBreakSpace"), 0x0a0 },
    { A("ExclamDown"), 0x0a1 },
    { A("Cent"), 0x0a2 },
    { A("Sterling"), 0x0a3 },
    { A("Currency"), 0x0a4 },
    { A("Yen"), 0x0a5 },
    { A("BrokenBar"), 0x0a6 },
    { A("Section"), 0x0a7 },
    { A("Diaeresis"), 0x0a8 },
    { A("Copyright"), 0x0a9 },
    { A("OrdFeminine"), 0x0aa },
    { A("GuillemotLeft"), 0x0ab },
    { A("NotSign"), 0x0ac },
    { A("Hyphen"), 0x0ad },
    { A("Registered"), 0x0ae },
    { A("Macron"), 0x0af },
    { A("Degree"), 0x0b0 },
    { A("PlusMinus"), 0x0b1 },
    { A("TwoSuperior"), 0x0b2 },
    { A("ThreeSuperior"), 0x0b3 },
    { A("Acute"), 0x0b4 },
    { A("Mu"), 0x0b5 },
    { A("Paragraph"), 0x0b6 },
    { A("PeriodCentered"), 0x0b7 },
    { A("Cedilla"), 0x0b8 },
    { A("OneSuperior"), 0x0b9 },
    { A("Masculine"), 0x0ba },
    { A("GuillemotRight"), 0x0bb },
    { A("OneQuarter"), 0x0bc },
    { A("OneHalf"), 0x0bd },
    { A("ThreeQuarters"), 0x0be },
    { A("QuestionDown"), 0x0bf },
    { A("Agrave"), 0x0c0 },
    { A("Aacute"), 0x0c1 },
    { A("Acircumflex"), 0x0c2 },
    { A("Atilde"), 0x0c3 },
    { A("Adiaeresis"), 0x0c4 },
    { A("Aring"), 0x0c5 },
    { A("AE"), 0x0c6 },
    { A("Ccedilla"), 0x0c7 },
    { A("Egrave"), 0x0c8 },
    { A("Eacute"), 0x0c9 },
    { A("Ecircumflex"), 0x0ca },
    { A("Ediaeresis"), 0x0cb },
    { A("Igrave"), 0x0cc },
    { A("Iacute"), 0x0cd },
    { A("Icircumflex"), 0x0ce },
    { A("Idiaeresis"), 0x0cf },
    { A("Eth"), 0x0d0 },
    { A("Ntilde"), 0x0d1 },
    { A("Ograve"), 0x0d2 },
    { A("Oacute"), 0x0d3 },
    { A("Ocircumflex"), 0x0d4 },
    { A("Otilde"), 0x0d5 },
    { A("Odiaeresis"), 0x0d6 },
    { A("Multiply"), 0x0d7 },
    { A("Ooblique"), 0x0d8 },
    { A("Ugrave"), 0x0d9 },
    { A("Uacute"), 0x0da },
    { A("Ucircumflex"), 0x0db },
    { A("Udiaeresis"), 0x0dc },
    { A("Yacute"), 0x0dd },
    { A("Thorn"), 0x0de },
    { A("SSharp"), 0x0df },
    { A("Division"), 0x0f7 },
    { A("YDiaeresis"), 0x0ff },
    { A("AltGr"), 0x01001103 },
    { A("Multi_key"), 0x01001120 },
    { A("Codeinput"), 0x01001137 },
    { A("SingleCandidate"), 0x0100113c },
    { A("MultipleCandidate"), 0x0100113d },
    { A("PreviousCandidate"), 0x0100113e },
    { A("Mode_switch"), 0x0100117e },
    { A("Script_Switch"), 0x0100117e },
    { A("Kanji"), 0x01001121 },
    { A("Muhenkan"), 0x01001122 },
    { A("Henkan_Mode"), 0x01001123 },
    { A("Henkan"), 0x01001123 },
    { A("Romaji"), 0x01001124 },
    { A("Hiragana"), 0x01001125 },
    { A("Katakana"), 0x01001126 },
    { A("Hiragana_Katakana"), 0x01001127 },
    { A("Zenkaku"), 0x01001128 },
    { A("Hankaku"), 0x01001129 },
    { A("Zenkaku_Hankaku"), 0x0100112a },
    { A("Touroku"), 0x0100112b },
    { A("Massyo"), 0x0100112c },
    { A("Kana_Lock"), 0x0100112d },
    { A("Kana_Shift"), 0x0100112e },
    { A("Eisu_Shift"), 0x0100112f },
    { A("Eisu_toggle"), 0x01001130 },
    { A("Kanji_Bangou"), 0x01001137 },
    { A("Zen_Koho"), 0x0100113d },
    { A("Mae_Koho"), 0x0100113e },
    { A("Hangul"), 0x01001131 },
    { A("Hangul_Start"), 0x01001132 },
    { A("Hangul_End"), 0x01001133 },
    { A("Hangul_Hanja"), 0x01001134 },
    { A("Hangul_Jamo"), 0x01001135 },
    { A("Hangul_Romaja"), 0x01001136 },
    { A("Hangul_Codeinput"), 0x01001137 },
    { A("Hangul_Jeonja"), 0x01001138 },
    { A("Hangul_Banja"), 0x01001139 },
    { A("Hangul_PreHanja"), 0x0100113a },
    { A("Hangul_PostHanja"), 0x0100113b },
    { A("Hangul_SingleCandidate"), 0x0100113c },
    { A("Hangul_MultipleCandidate"), 0x0100113d },
    { A("Hangul_PreviousCandidate"), 0x0100113e },
    { A("Hangul_Special"), 0x0100113f },
    { A("Hangul_switch"), 0x0100117e },
    { A("Dead_Grave"), 0x01001250 },
    { A("Dead_Acute"), 0x01001251 },
    { A("Dead_Circumflex"), 0x01001252 },
    { A("Dead_Tilde"), 0x01001253 },
    { A("Dead_Macron"), 0x01001254 },
    { A("Dead_Breve"), 0x01001255 },
    { A("Dead_Abovedot"), 0x01001256 },
    { A("Dead_Diaeresis"), 0x01001257 },
    { A("Dead_Abovering"), 0x01001258 },
    { A("Dead_Doubleacute"), 0x01001259 },
    { A("Dead_Caron"), 0x0100125a },
    { A("Dead_Cedilla"), 0x0100125b },
    { A("Dead_Ogonek"), 0x0100125c },
    { A("Dead_Iota"), 0x0100125d },
    { A("Dead_Voiced_Sound"), 0x0100125e },
    { A("Dead_Semivoiced_Sound"), 0x0100125f },
    { A("Dead_Belowdot"), 0x01001260 },
    { A("Dead_Hook"), 0x01001261 },
    { A("Dead_Horn"), 0x01001262 },
    { A("Back"), 0x01000061 },
    { A("Forward"), 0x01000062 },
    { A("Stop"), 0x01000063 },
    { A("Refresh"), 0x01000064 },
    { A("VolumeDown"), 0x01000070 },
    { A("VolumeMute"), 0x01000071 },
    { A("VolumeUp"), 0x01000072 },
    { A("BassBoost"), 0x01000073 },
    { A("BassUp"), 0x01000074 },
    { A("BassDown"), 0x01000075 },
    { A("TrebleUp"), 0x01000076 },
    { A("TrebleDown"), 0x01000077 },
    { A("MediaPlay"), 0x01000080 },
    { A("MediaStop"), 0x01000081 },
    { A("MediaPrevious"), 0x01000082 },
    { A("MediaNext"), 0x01000083 },
    { A("MediaRecord"), 0x01000084 },
    { A("MediaPause"), 0x1000085 },
    { A("MediaTogglePlayPause"), 0x1000086 },
    { A("HomePage"), 0x01000090 },
    { A("Favorites"), 0x01000091 },
    { A("Search"), 0x01000092 },
    { A("Standby"), 0x01000093 },
    { A("OpenUrl"), 0x01000094 },
    { A("LaunchMail"), 0x010000a0 },
    { A("LaunchMedia"), 0x010000a1 },
    { A("Launch0"), 0x010000a2 },
    { A("Launch1"), 0x010000a3 },
    { A("Launch2"), 0x010000a4 },
    { A("Launch3"), 0x010000a5 },
    { A("Launch4"), 0x010000a6 },
    { A("Launch5"), 0x010000a7 },
    { A("Launch6"), 0x010000a8 },
    { A("Launch7"), 0x010000a9 },
    { A("Launch8"), 0x010000aa },
    { A("Launch9"), 0x010000ab },
    { A("LaunchA"), 0x010000ac },
    { A("LaunchB"), 0x010000ad },
    { A("LaunchC"), 0x010000ae },
    { A("LaunchD"), 0x010000af },
    { A("LaunchE"), 0x010000b0 },
    { A("LaunchF"), 0x010000b1 },
    { A("MonBrightnessUp"), 0x010000b2 },
    { A("MonBrightnessDown"), 0x010000b3 },
    { A("KeyboardLightOnOff"), 0x010000b4 },
    { A("KeyboardBrightnessUp"), 0x010000b5 },
    { A("KeyboardBrightnessDown"), 0x010000b6 },
    { A("PowerOff"), 0x010000b7 },
    { A("WakeUp"), 0x010000b8 },
    { A("Eject"), 0x010000b9 },
    { A("ScreenSaver"), 0x010000ba },
    { A("WWW"), 0x010000bb },
    { A("Memo"), 0x010000bc },
    { A("LightBulb"), 0x010000bd },
    { A("Shop"), 0x010000be },
    { A("History"), 0x010000bf },
    { A("AddFavorite"), 0x010000c0 },
    { A("HotLinks"), 0x010000c1 },
    { A("BrightnessAdjust"), 0x010000c2 },
    { A("Finance"), 0x010000c3 },
    { A("Community"), 0x010000c4 },
    { A("AudioRewind"), 0x010000c5 },
    { A("BackForward"), 0x010000c6 },
    { A("ApplicationLeft"), 0x010000c7 },
    { A("ApplicationRight"), 0x010000c8 },
    { A("Book"), 0x010000c9 },
    { A("CD"), 0x010000ca },
    { A("Calculator"), 0x010000cb },
    { A("ToDoList"), 0x010000cc },
    { A("ClearGrab"), 0x010000cd },
    { A("Close"), 0x010000ce },
    { A("Copy"), 0x010000cf },
    { A("Cut"), 0x010000d0 },
    { A("Display"), 0x010000d1 },
    { A("DOS"), 0x010000d2 },
    { A("Documents"), 0x010000d3 },
    { A("Excel"), 0x010000d4 },
    { A("Explorer"), 0x010000d5 },
    { A("Game"), 0x010000d6 },
    { A("Go"), 0x010000d7 },
    { A("iTouch"), 0x010000d8 },
    { A("LogOff"), 0x010000d9 },
    { A("Market"), 0x010000da },
    { A("Meeting"), 0x010000db },
    { A("MenuKB"), 0x010000dc },
    { A("MenuPB"), 0x010000dd },
    { A("MySites"), 0x010000de },
    { A("News"), 0x010000df },
    { A("OfficeHome"), 0x010000e0 },
    { A("Option"), 0x010000e1 },
    { A("Paste"), 0x010000e2 },
    { A("Phone"), 0x010000e3 },
    { A("Calendar"), 0x010000e4 },
    { A("Reply"), 0x010000e5 },
    { A("Reload"), 0x010000e6 },
    { A("RotateWindows"), 0x010000e7 },
    { A("RotationPB"), 0x010000e8 },
    { A("RotationKB"), 0x010000e9 },
    { A("Save"), 0x010000ea },
    { A("Send"), 0x010000eb },
    { A("Spell"), 0x010000ec },
    { A("SplitScreen"), 0x010000ed },
    { A("Support"), 0x010000ee },
    { A("TaskPane"), 0x010000ef },
    { A("Terminal"), 0x010000f0 },
    { A("Tools"), 0x010000f1 },
    { A("Travel"), 0x010000f2 },
    { A("Video"), 0x010000f3 },
    { A("Word"), 0x010000f4 },
    { A("Xfer"), 0x010000f5 },
    { A("ZoomIn"), 0x010000f6 },
    { A("ZoomOut"), 0x010000f7 },
    { A("Away"), 0x010000f8 },
    { A("Messenger"), 0x010000f9 },
    { A("WebCam"), 0x010000fa },
    { A("MailForward"), 0x010000fb },
    { A("Pictures"), 0x010000fc },
    { A("Music"), 0x010000fd },
    { A("Battery"), 0x010000fe },
    { A("Bluetooth"), 0x010000ff },
    { A("WLAN"), 0x01000100 },
    { A("UWB"), 0x01000101 },
    { A("AudioForward"), 0x01000102 },
    { A("AudioRepeat"), 0x01000103 },
    { A("AudioRandomPlay"), 0x01000104 },
    { A("Subtitle"), 0x01000105 },
    { A("AudioCycleTrack"), 0x01000106 },
    { A("Time"), 0x01000107 },
    { A("Hibernate"), 0x01000108 },
    { A("View"), 0x01000109 },
    { A("TopMenu"), 0x0100010a },
    { A("PowerDown"), 0x0100010b },
    { A("Suspend"), 0x0100010c },
    { A("ContrastAdjust"), 0x0100010d },
    { A("LaunchG"), 0x0100010e },
    { A("LaunchH"), 0x0100010f },
    { A("TouchpadToggle"), 0x01000110 },
    { A("TouchpadOn"), 0x01000111 },
    { A("TouchpadOff"), 0x01000112 },
    { A("MicMute"), 0x01000113 },
    { A("Red"), 0x01000114 },
    { A("Green"), 0x01000115 },
    { A("Yellow"), 0x01000116 },
    { A("Blue"), 0x01000117 },
    { A("ChannelUp"), 0x01000118 },
    { A("ChannelDown"), 0x01000119 },
    { A("Guide"), 0x0100011a },
    { A("Info"), 0x0100011b },
    { A("Settings"), 0x0100011c },
    { A("MicVolumeUp"), 0x0100011d },
    { A("MicVolumeDown"), 0x0100011e },
    { A("New"), 0x01000120 },
    { A("Open"), 0x01000121 },
    { A("Find"), 0x01000122 },
    { A("Undo"), 0x01000123 },
    { A("Redo"), 0x01000124 },
    { A("MediaLast"), 0x0100ffff },
    { A("Select"), 0x01010000 },
    { A("Yes"), 0x01010001 },
    { A("No"), 0x01010002 },
    { A("Cancel"), 0x01020001 },
    { A("Printer"), 0x01020002 },
    { A("Execute"), 0x01020003 },
    { A("Sleep"), 0x01020004 },
    { A("Play"), 0x01020005 },
    { A("Zoom"), 0x01020006 },
    { A("Jisho"), 0x01020007 },
    { A("Oyayubi_Left"), 0x01020008 },
    { A("Oyayubi_Right"), 0x01020009 },
    { A("Exit"), 0x0102000a },
    { A("Context1"), 0x01100000 },
    { A("Context2"), 0x01100001 },
    { A("Context3"), 0x01100002 },
    { A("Context4"), 0x01100003 },
    { A("Call"), 0x01100004 },
    { A("Hangup"), 0x01100005 },
    { A("Flip"), 0x01100006 },
    { A("ToggleCallHangup"), 0x01100007 },
    { A("VoiceDial"), 0x01100008 },
    { A("LastNumberRedial"), 0x01100009 },
    { A("Camera"), 0x01100020 },
    { A("CameraFocus"), 0x01100021 },

    // Buttons
    { A("BackButton"), 0x40000008 },
    { A("ForwardButton"), 0x40000010 },
    { A("TaskButton"), 0x40000020 },
    { A("MouseButton4"), 0x40000040 },
    { A("MouseButton5"), 0x40000080 },
    { A("MouseButton6"), 0x40000100 },
    { A("MouseButton7"), 0x40000200 },
    { A("MouseButton8"), 0x40000400 },
    { A("MouseButton9"), 0x40000800 },
    { A("MouseButton10"), 0x40001000 },
    { A("MouseButton11"), 0x40002000 },
    { A("MouseButton12"), 0x40004000 },
    { A("MouseButton13"), 0x40008000 },
    { A("MouseButton14"), 0x40010000 },
    { A("MouseButton15"), 0x40020000 },
    { A("MouseButton16"), 0x40040000 },
    { A("MouseButton17"), 0x40080000 },
    { A("MouseButton18"), 0x40100000 },
    { A("MouseButton19"), 0x40200000 },
    { A("MouseButton20"), 0x40400000 },
    { A("MouseButton21"), 0x40800000 },
    { A("MouseButton22"), 0x41000000 },
    { A("MouseButton23"), 0x42000000 },
    { A("MouseButton24"), 0x44000000 },
};

const QStringList& sortedKeyNames()
{
    static bool haveResult;
    static QStringList result;

    if (!haveResult) {
        result = g_keyNames.keys();
        result.sort();
        haveResult = true;
    }

    return result;
}
